use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/HTML/Latemp/GenMakeHelpers.pm',
    't/00-compile.t',
    't/01-output-basic.t',
    't/sample-data/common-1/foo.js',
    't/sample-data/sample-site-1/alternatives/index.html.wml',
    't/sample-data/sample-site-1/arch/index.html.wml',
    't/sample-data/sample-site-1/docs/index.html.wml',
    't/sample-data/sample-site-1/docs/nice_trys.html.wml',
    't/sample-data/sample-site-1/docs/shlomif-evolution.html.wml',
    't/sample-data/sample-site-1/images/arrow-left-disabled.png',
    't/sample-data/sample-site-1/images/arrow-left.png',
    't/sample-data/sample-site-1/images/arrow-right-disabled.png',
    't/sample-data/sample-site-1/images/arrow-right.png',
    't/sample-data/sample-site-1/images/arrow-up-disabled.png',
    't/sample-data/sample-site-1/images/arrow-up.png',
    't/sample-data/sample-site-1/images/berlios-logo.png',
    't/sample-data/sample-site-1/images/better-scm-logo.png',
    't/sample-data/sample-site-1/images/get-firefox.png',
    't/sample-data/sample-site-1/images/logo-wml.png',
    't/sample-data/sample-site-1/images/somerights20.png',
    't/sample-data/sample-site-1/images/valid-css.png',
    't/sample-data/sample-site-1/images/valid-xhtml10.png',
    't/sample-data/sample-site-1/images/valid-xhtml11.png',
    't/sample-data/sample-site-1/index.html.wml',
    't/sample-data/sample-site-1/irc/index.html.wml',
    't/sample-data/sample-site-1/links.html.wml',
    't/sample-data/sample-site-1/mailing-list.html.wml',
    't/sample-data/sample-site-1/print.css',
    't/sample-data/sample-site-1/site-map/index.html.wml',
    't/sample-data/sample-site-1/source/index.html.wml',
    't/sample-data/sample-site-1/style.css',
    't/sample-data/sample-site-1/subversion/Subversion-Win32-Installation-Guide.txt',
    't/sample-data/sample-site-1/subversion/Svn-Win32-Inst-Guide.html.wml',
    't/sample-data/sample-site-1/subversion/compelling_alternative.html.wml',
    't/sample-data/sample-site-1/subversion/index.html.wml'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
